local type = type
local L = LibStub("AceLocale-3.0"):GetLocale("Dock")
setfenv(1, Dock.FunctionEnviroment)

do
	local animations = {None = false,}
	local animationList = {None = "No animation",}
	function Core:AddAnimation(name, desc, func)
		if type(name) ~= "string" then
			error("bad argument #2 to AddAnimation' (string expected, got %s)", type(name))
		end
		if type(desc) ~= "string" then
			error("bad argument #3 to AddAnimation' (string expected, got %s)", type(desc))
		end
		if type(func) ~= "function" then
			error("bad argument #4 to AddAnimation' (function expected, got %s)", type(func))
		end
		animations[name] = func
		animationList[name] = desc
	end
	function Core:GetAnimationList()
		return animationList
	end
	function Core:GetAnimation(name)
		return animations[name]
	end
end

Core:AddAnimation(
	"CurveUpPower",
	"Curves up, starts fast then slows",
	function(currentx, currenty, endingx, endingy, rate)
		rate = rate * .8
		local xDiff = endingx - currentx
		local offset = (xDiff > 0 and xDiff or -xDiff) ^ .1
		return currentx + xDiff * rate, currenty + ((endingy * (offset > 1 and offset or 1)) - currenty) * rate
	end
)

Core:AddAnimation(
	"CurveUpConstant",
	"Curves up, constant speed", 
	function(currentx, currenty, endingx, endingy, rate)
		local xDiff = endingx - currentx
		local yDiff = endingy - currenty

		local x
		local change = rate * 50
		if (xDiff < 0 and -xDiff or xDiff) > change then
			if endingx > currentx then
				x = currentx + change
			else
				x = currentx - change
			end
		else
			x = endingx
		end
		local offset = ((xDiff > 0 and xDiff or -xDiff) ^ .5) * .07
		return x, currenty + ((endingy * (offset > 1 and offset or 1)) - currenty) * rate
	end
)

Core:AddAnimation(
	"CurveDownPower", 
	"Curves down, starts fast then slows", 
	function(currentx, currenty, endingx, endingy, rate)
		rate = rate * .8
		local xDiff = endingx - currentx
		local offset = (xDiff > 0 and xDiff or -xDiff) ^ .1
		return currentx + xDiff * rate, currenty + ((endingy / (offset > 1 and offset or 1)) - currenty) * rate
	end
)

Core:AddAnimation(
	"CurveDownConstant", 
	"Curves down, constant speed", 
	function(currentx, currenty, endingx, endingy, rate)
		local xDiff = endingx - currentx
		local yDiff = endingy - currenty

		local x
		local change = rate * 50
		if (xDiff < 0 and -xDiff or xDiff) > change then
			if endingx > currentx then
				x = currentx + change
			else
				x = currentx - change
			end
		else
			x = endingx
		end
		local offset = ((xDiff > 0 and xDiff or -xDiff) ^ .5) * .07
		return x, currenty + ((endingy / (offset > 1 and offset or 1)) - currenty) * rate
	end
)

Core:AddAnimation(
	"XThenY", 
	"Right-angle, finishing X movement first", 
	function(currentx, currenty, endingx, endingy, rate)
		local xDiff = endingx - currentx
		local yDiff = endingy - currenty

		local x, y
		local change = rate * 50
		if (xDiff < 0 and -xDiff or xDiff) > change then
			if endingx > currentx then
				x = currentx + change
			else
				x = currentx - change
			end
		else
			x = endingx
		end
		if x == endingx then
			if (yDiff < 0 and -yDiff or yDiff) > change then
				if endingy > currenty then
					y = currenty + change
				else
					y = currenty - change
				end
			else
				y = endingy
			end
		else
			y = currenty
		end
		
		return x, y
	end
)

Core:AddAnimation(
	"YThenX", 
	"Right-angle, finishing Y movement first", 
	function (currentx, currenty, endingx, endingy, rate)
		local xDiff = endingx - currentx
		local yDiff = endingy - currenty

		local x, y
		local change = rate * 50
		
		if (yDiff < 0 and -yDiff or yDiff) > change then
			if endingy > currenty then
				y = currenty + change
			else
				y = currenty - change
			end
		else
			y = endingy
		end
		if y == endingy then
			if (xDiff < 0 and -xDiff or xDiff) > change then
				if endingx > currentx then
					x = currentx + change
				else
					x = currentx - change
				end
			else
				x = endingx
			end
		else
			x = currentx
		end
		
		return x, y
	end
)

Core:AddAnimation(
	"SlidePower", 
	"Slide, starts fast then slows", 
	function(currentx, currenty, endingx, endingy, rate)
		rate = rate * .1
		local min = rate * 100
		local xDiff = (endingx - currentx) * rate
		local yDiff = (endingy - currenty) * rate

		local x, y
		if xDiff > 0 then
			if (endingx - currentx) < min then
				x = endingx
			else
				x = currentx + (xDiff > min and xDiff or min)
			end
		else
			if (endingx - currentx) > -min then
				x = endingx
			else
				x = currentx + (xDiff < -min and xDiff or -min)
			end
		end
		if yDiff > 0 then
			if (endingy - currenty) < min then
				y = endingy
			else
				y = currenty + (yDiff > min and yDiff or min)
			end
		else
			if (endingy - currenty) > -min then
				y = endingy
			else
				y = currenty + (yDiff < -min and yDiff or -min)
			end
		end
		return x, y
	end
)

Core:AddAnimation(
	"SlideConstant", 
	"Slide, constant speed", 
	function(currentx, currenty, endingx, endingy, rate)
		local xDiff = endingx - currentx
		local yDiff = endingy - currenty

		local x, y
		local change = rate * 50
		if (xDiff < 0 and -xDiff or xDiff) > change then
			if endingx > currentx then
				x = currentx + change
			else
				x = currentx - change
			end
		else
			x = endingx
		end
		if (yDiff < 0 and -yDiff or yDiff) > change then
			if endingy > currenty then
				y = currenty + change
			else
				y = currenty - change
			end
		else
			y = endingy
		end
		return x, y
	end
)